;		TITLE  'HB100 3cm FREQ COUNTER DECADE 15/4/2017'			
;		V1	using dsPIC30F4012 Micro
;		V2 adding a decade 100KHz and GPS gate timer 
;		V3 changed to a dsPIC30F2010 u/P 22/4/2017
;-------------------------------------------------------
;	       	         	28 PIN DIL 
;		            	dsPIC30F2010
;                     ______  ______
;		    	     |      \/      |
;   PROGRAMER/MCLR --+ 1         28 +-- RB7  AVdd	
;	            	 |              |
;              RB0 --+ 2         27 +-- RB6  AVss
;	            	 |              |
;              RB1 --+ 3         26 +-- RE0  		
;	           	     |              |
;              RB2 --+ 4         25 +-- RE1  
;	                 |              |
;              RB3 --+ 5         24 +-- RE2	             |
;	                 |              |
;              RB4 --+ 6         23 +-- RE3
;	                 |              |
;              RB5 --+ 7         22 +-- RE4
;	                 |              |
;              VSS  -+ 8         21 +-- RE5
;	                 |              |
;              OSC --+ 9         20 +-- VDD
;	                 |              |
;             OSC1 --+ 10        19 +-- VSS
;	                 |              |
;         GPS/RC13 --+ 11        18 +-- RF2/PGC	
;	                 |              |
; INPUT HB100/RC14 --+ 12        17 +-- RF3/PGD
;	                 |              |
;              VDD --+ 13        16 +-- RE8/SDA DISPLAY
;	                 |              |
;              RD1 --+ 14        15 +-- RD0/SCL DISPLAY
;                    |______________|
;
;		            	PCF8574
;                     ______  ______
;		             |      \/      |
;               A0 --+ 1         16 +-- Vcc	
;	                 |              |
;               A1 --+ 2         15 +-- SDA
;	                 |              |
;               A2 --+ 3         14 +-- SCL		
;	                 |              |
;            RS/P0 --+ 4         13 +-- /INT
;	                 |              |
;    	     RW/P1 --+ 5         12 +-- P7/DB7
;	                 |              |
;   	      E/P2 --+ 6         11 +-- P6/DB6
;	                 |              |
;               P3 --+ 7         10 +-- P5/DB5
;	                 |              |
;              GND  -+ 8          9 +-- P4/DB4
;	                 |______________|
;					
;					LCD DISPLAY
;					  ______  
;	                 | GND  |
;	                 | VCC  |
;	                 | SDA  |
;	                 | SCL  |
;	                 |______|	  
;-----------------------------------------------------------------------------
       .equ __30F2010, 1
        .include "p30f2010.inc"
;.............................................................................
;Program Specific Constants (literals used in code)
;.............................................................................
;-------------- global declarations ---------------------------------------
	.global __reset
	.global __T1Interrupt
;.............................................................................
;Constants stored in Program space
;.............................................................................
        .section .myconstbuffer, psv
;.............................................................................
;Code Section in Program Memory
;-----------------------------------------------------------------------------
;Configuration bits:
	config __FPR_PLL4
	config __FOSC XT
	config __FWDT,WDT_OFF
	config __PBORPOR, PBOR_OFF & MCLR_EN
	config __FGS, CODE_PROT_OFF
;-----------------------------------------------------------------------------        
;define constant
;PORTB
;PORTC
	.equ	gps,#13
;PORTD
	.equ	scl,#0
;PORTE
	.equ	sda,#8
;-------------------------------------------
;define ram assignments
	.bss	Lin1,2
	.bss	Lin2,2
	.bss	Lin3,2
	.bss	Lin4,2
	.bss	Lin5,2
	.bss	Lin6,2
	.bss	Lin7,2
	.bss	Lin8,2
	.bss	Lin9,2
	.bss	Lin10,2
	.bss	Lin11,2
	.bss	Lin12,2
	.bss	Lin13,2
	.bss	Lin14,2
	.bss	Lin15,2
	.bss	Lin16,2
	.bss	Lin17,2
	.bss	Lin18,2
	.bss	Lin19,2
	.bss	Lin20,2
	.bss	Lin21,2
	.bss	Lin22,2
	.bss	Lin23,2
	.bss	Lin24,2
	.bss	Lin25,2
	.bss	Lin26,2
	.bss	Lin27,2
	.bss	Lin28,2
	.bss	Lin29,2
	.bss	Lin30,2
	.bss	Lin31,2
	.bss	Lin32,2

	.bss	upper,2
	.bss	lower,2
	.bss	count,2
	.bss	block,2
	.bss	bits,2
	.bss	datbuff,2
	.bss	pointer_numb,2
	.bss	temp1,2
	.bss	temp2,2
	.bss	digit1,2	
	.bss	digit2,2	
	.bss	digit3,2	
	.bss	digit4,2
	.bss	digit5,2
	.bss	digit6,2
	.bss	digit7,2
	.bss	digit8,2
	.bss	offset,2
	.bss	gate,2
;---------------------------------------------------
	.text				;start of code
__reset:
    mov 	#__SP_init, W15       	;Initalize the Stack Pointer
    mov 	#__SPLIM_init, W0     	;Initialize the Stack Pointer Limit Register
    mov 	W0, SPLIM
    nop                     	    ;Add NOP to follow SPLIM initialization
	mov		#0xffff,w0		
	mov		w0,ADPCFG	         	;all digital I/O	
	clr		LATB
	clr		LATC
	clr		LATD
	clr		LATE	                                                                                                                               
;----------------------- INITIALISATION ----------------
init:
	call	init_ports
	call	init_lcd
	call	LCD_Line1
	mov		#0x1388,w0	;(dec 5,000)
	mov		w0,offset
	call	unpack
	call	dec_hex_conv
	call	send_buff	
	call	init_tmr1	
	goto	main
;--------------------------------------------------
 __T1Interrupt:
 	dec		offset
 	bclr	IFS0,#T1IF
 	retfie		
;---------------------- MAIN ROUTINE ------------------
main:
	btst	PORTC,#gps	;ave we a GPS 1 SEC PULSE
	bra		z,main		;No	
pulse:
	btst	PORTC,#gps	;ave we a GPS 1 SEC PULSE
	bra		z,load		;No
	goto	pulse
load:	
	call	LCD_Line1	;Yes
	call	unpack
	call	dec_hex_conv
	call	send_buff
	mov		#0x1388,w0	;reprime the counter 5000
	mov		w0,offset		
	goto	main
;---------------------------------------------------
;	SUBROUTINES
unpack:
	clr		digit1
	inc		digit1
	clr		digit2
	mov		offset,w1
	mov		#1000,w2
	repeat	#17
	div.u	w1,w2
	mov		w0,digit3
	mov		#100,w2
	repeat	#17
	div.u	w1,w2
	mov		w0,digit4
	mov		#10,w2
	repeat	#17
	div.u	w1,w2
	mov		w0,digit5
	mov		w1,digit6
	return
;--------------------------------------------------------
dec_hex_conv:	
	mov		#0x20,w0
	mov		w0,Lin1		;" "
	mov		#0x20,w0
	mov		w0,Lin2		;" "	
	mov		digit1,w0
	add		#0x30,w0
	mov		w0,Lin3	
	mov		digit2,w0	
	add		#0x30,w0	
	mov		w0,Lin4	
	mov		#0x2c,w0
	mov		w0,Lin5		;","	
	mov		digit3,w0
	add		#0x30,w0	
	mov		w0,Lin6
	mov		digit4,w0
	add		#0x30,w0	
	mov		w0,Lin7
	mov		digit5,w0
	add		#0x30,w0	
	mov		w0,Lin8
	mov		#0x2e,w0
	mov		w0,Lin9		;"."
	mov		digit6,w0
	add		#0x30,w0
	mov		w0,Lin10
	mov		#0x20,w0
	mov		w0,Lin11	;" "	
	mov	    #0x4d,w0
	mov		w0,Lin12	;"M"
	mov		#0x48,w0
	mov		w0,Lin13	;"H"
	mov		#0x7a,w0
	mov		w0,Lin14	;"z"
	mov		#0x20,w0
	mov		w0,Lin15	;" "
	mov		#0x20,w0
	mov		w0,Lin16	;" "
	return
;-------------------------------------------------------
send_8:
	mov		#0x08,w0
	mov		w0,bits
loop8:
	bclr	LATD,#sda
	btst	datbuff,#0
	bra		z,zero
	bset	LATD,#sda
zero:
	call	w_clk
	lsr		datbuff
	dec		bits
	bra		nz,loop8	
	return
;--------------------------------------------------
w_clk:
	bset	LATD,#0
	repeat	#100
	nop
	bclr	LATD,#0
	repeat	#100
	nop
	return
;------------------ INIT LCD START ADDRESS -------------------
LCD_Line1:
	mov		#0x80,w0	;send to control register
	call	bld_command
	return
LCD_Line2:
	mov		#0x94,w0	;send to control register
	call	bld_command
	return
LCD_Line3:
	mov		#0xa8,w0	;send to control register
	call	bld_command
	return
LCD_Line4:
	mov		#0xbc,w0	;send to control register
	call	bld_command
	return
;------------------ INIT LCD SETTUP -------------------
init_lcd_tab:
	bra		w0	
	retlw	#0x38,w0	;0
	retlw	#0x38,w0	;1		;30
	retlw	#0x04,w0	;		command
	retlw	#0x08,w0	;		turn lcd on	;08
	retlw	#0x0c,w0	;4		;02
	retlw	#0x00,w0	;5
	retlw	#0x24,w0	;6	
	retlw	#0x00,w0	;		sets up two line operation
;------------------------------------------------------
i2c_clk_LCD:
	bset	LATE,#scl
	call	I2C_dly
	nop
	nop
	bclr	LATD,#scl
	return
;------------------------- small delay ------------------
I2C_dly:
	repeat	#1000
	nop
	return
;--------------------------------------------------------
init_tmr1:
	mov		#0x8012,w0
	mov		w0,T1CON
	mov		#0x00c3,w0	;195
	mov		w0,PR1
	clr		TMR1
 	bclr	IFS0,#T1IF
 	bset	IPC0,#14
 	bclr	IPC0,#13
 	bclr	IPC0,#12
 	bset	IEC0,#T1IE
	return
;----------------------------------------------------
init_lcd:
;		01	REGISTER SELECT
;		02	READ/WRITE SELECT
;		04	ENABLE BIT
	call	start_LCD
	call	header_LCD	;send i2c address "4e"
	mov		#0x0000,w0
	call	init_lcd_tab 
	mov		w0,datbuff	;set DL to be 4 bit
	call	sendlcd
	mov		#0x0001,w0
	call	init_lcd_tab 
	mov		w0,datbuff	;1
	call	sendlcd
	mov		#0x0002,w0
	call	init_lcd_tab 
	mov		w0,datbuff	;2
	call	sendlcd
	mov		#0x0003,w0
	call	init_lcd_tab 
	mov		w0,datbuff	;3
	call	sendlcd
	mov		#0x0004,w0
	call	init_lcd_tab 
	mov		w0,datbuff	;4
	call	sendlcd
	mov		#0x0005,w0
	call	init_lcd_tab 
	mov		w0,datbuff	;5
	call	sendlcd
	mov		#0x0006,w0
	call	init_lcd_tab 
	mov		w0,datbuff	;6
	call	sendlcd
	mov		#0x0007,w0
	call	init_lcd_tab 
	mov		w0,datbuff	;7
	call	sendlcd
	return
;------------------------------
bld_byte:
	push	w0
	mov		w0,upper
	sl		w0,#4,w0
	mov		w0,lower
	bset	lower,#0
	bclr	lower,#1
	bset	lower,#2
	bset	lower,#3		
	bset	upper,#0
	goto	ext
;------------------------------
bld_command:
	push	w0
	mov		w0,upper
	sl		w0,#4,w0
	mov		w0,lower
	bclr	lower,#0
	bclr	lower,#1
	bset	lower,#2
	bset	lower,#3		
	bclr	upper,#0
ext:
	bclr	upper,#1
	bset	upper,#2
	bset	upper,#3
	mov		upper,w0
	mov		w0,datbuff
	call	sendlcd
	bclr	upper,#2
	mov		upper,w0
	mov		w0,datbuff
	call	sendlcd
	mov		lower,w0
	mov		w0,datbuff
	call	sendlcd
	bclr	lower,#2
	mov		lower,w0
	mov		w0,datbuff
	call	sendlcd	
	pop		w0
	return
;------------------------------
header_LCD:
	mov		#0x4e,w0	;lcd address
	mov		w0,datbuff
	call	sendlcd		;send address to i2c chip
	return
;---------------------------------
sendlcd:
	mov		datbuff,w0	;saved data
	mov		w0,temp2	;this will get destroyed
	mov		#0x08,w0	;send 8 bits
	mov		w0,temp1
send8_i2c:
	bclr	LATE,#sda
	btsc	temp2,#0x07
	bset	LATE,#sda	;set data bit	
	sl		temp2	
	call	i2c_clk
	dec		temp1
	bra		nz,send8_i2c
	bset	LATE,#sda
	call	i2c_clk		;ack bit clock
	return
;----------------------
start_LCD:
	bclr	LATE,#sda
	nop
	nop
	nop
	bclr	LATD,#scl	;start condition
	return
;----------------------
stop_LCD:
	bset	LATD,#scl
	nop
	nop
	nop
	bset	LATE,#sda	;stop condition
	return
;---------------------------------------------------------
i2c_clk:
	bset	LATD,#scl
	nop
	nop
	nop
	nop
	nop
	nop
	bclr	LATD,#scl	;synth clock pulse
	return
;------------------------ load buffers to display ram --------
send_buff:
	call	start_LCD
	mov		Lin1,w0
	call	bld_byte
	mov		Lin2,w0
	call	bld_byte
	mov		Lin3,w0
	call	bld_byte
	mov		Lin4,w0
	call	bld_byte
	mov		Lin5,w0
	call	bld_byte
	mov		Lin6,w0
	call	bld_byte
	mov		Lin7,w0
	call	bld_byte
	mov		Lin8,w0
	call	bld_byte
	mov		Lin9,w0
	call	bld_byte
	mov		Lin10,w0
	call	bld_byte
	mov		Lin11,w0
	call	bld_byte
	mov		Lin12,w0
	call	bld_byte
	mov		Lin13,w0
	call	bld_byte
	mov		Lin14,w0
	call	bld_byte
	mov		Lin15,w0
	call	bld_byte
	mov		Lin16,w0
	call	bld_byte
	return
;--------------------------- SET UP PORTS -----------------
init_ports:
	clr		PTCON
	clr		PWMCON1
	clr		OVDCON
	clr		LATB
	mov		#0x6000,w0	;0=output
	mov		w0,TRISC	;sets HB100 and GPS has inputs
	mov		#0x0000,w0
	mov		w0,TRISB	;1=input
	mov		#0x0000,w0
	mov		w0,TRISD
	mov		#0x0080,w0
	mov		w0,TRISE
	call	stop_LCD
	return
;---------------- display update delay ----------
update_delay:
	mov	#0x0070,w3	
big:
	repeat	#16000
	nop
	dec		w3,w3
	bra		nz,big
	inc		digit6
	return
;--------------------------------------------
	.end




